; PUSH - a simple testbed SOKOBAN clone
; by Andrew Davie May 1998

; I'm releasing this code to the public, in the interests of sharing
; knowledge with my fellow developers.  If you use some of the code here,
; fine.  A bit of acknowledgement would be nice.

; If you hate things you see, remember that it's a TEST program.  I'm
; learning about the machine by trying out different things.  I'm interested
; in hearing about BUGS, but I'm definitely NOT interested in hearing
; criticisms about my programming style :)

; There are lots of things I'd change in this code - specifically the way
; the playfield hogs RAM.  I switched to dual playfelds when I had some
; timing problems - earlier 4-colour playfield code used a write/erase
; strategy and a single playfield.  You may note that there are NO sprites
; in use in this program - at the stage this project was effectively
; abandoned, I was gearing up my playfield animation / manipulation
; skills.

; So, remember its free.  Play around, enjoy!

    processor 6502
    include vcs.h


TOTLINES        = 192       ; adjust this value to set screen lines
MOVEDELAY       = 40        ; speed of block pushing
OBJSIZE         = 6         ; # blocks supported
PFSIZE          = 5*9       ; 5 bytes / line by n lines

; Notes:

; Although we have 40 pixels across the screen (6 bytes for display), we
; only use 32 of these as actual gameplay area, requiring just 32 bits
; per line for each information segment being stored. The compressed
; playfield representations are a logical ordered representation of
; playfield positions, and do not directly correspond to the playfield
; table, which is used for display purposes.  That is, indexing into
; playfield table must take account of PF0/1/2 mirroring, etc.


; bits
D0 = 1
D1 = 2
D2 = 4
D3 = 8
D4 = 16
D5 = 32
D6 = 64
D7 = 128


    
;-------
; variables

    SEG.U vars
    org $80

    ; only 128 bytes... including the stack!
    ; let the assembler calculate the addresses

    ; really an inefficient hodge-podge of temporary variables at the
    ; moment.  Sure, this could be tidied up - go for it :)

movespeed       ds 1
targettoggle    ds 1
targetmask      ds 2
temp            ds 2
toggle          ds 1
level           ds 2            ; pointer to current level table
count           ds 1            ; general loop counter
currenttarget   ds 1
timer           ds 1
newpos          ds 2

    ; the playfield stuff is gonna change.. don't complain about memory!
    ; Two playfields now - corresponding to alternate frames in the 
    ; multiplexed 4-colour display.  Earlier version of 4CPF used a single
    ; frame buffer, with realtime writes/erases according to colour.

playfield       ds PFSIZE
playfield2      ds PFSIZE

objpos          ds OBJSIZE
    ; d7        active (1 = inactive)
    ; d6-d0     cell number for object
    
objflags    ds OBJSIZE
    ; d7        force draw
    ; d0-d1     colour

;--------

    SEG code
    org $F000
; eek! just 4k (-vectors)
       
;--------
; start
; program entry point

start
        sei
        cld

    ; clears all variables, stack
    ; init stack pointer
    ; also clears tia registers
    ; does this by "wrapping" the stack - unusual

    ; don't place this in a subroutine - you dunno what the stack is!

        ldx #0
        txs
        pha                     ; best way to get sp=$ff, x=0

        txa
clear   pha
        dex
        bne clear

    ; 9 bytes total for clearing stack, memory
    ; stack pointer now $ff, a=x==0

        jsr init1ce             ; system reset inits

    ; main loop - this is the whole shebang

main    jsr verticalblank       ; start the display
        jsr overscan
        jmp main

;--------
; verticalblank

    ; writes the vertical synch signal for the start of a frame
    ; sets up timer so we can keep track of available time - note
    ; there are 37 lines available before the screen needs to start
    ; drawing - approx 2800 cycles therefore available for game
    ; calculations

verticalblank

    ; typical frame consists of 3 vertical sync (VSYNC) lines,
    ; 37 vertical blank (vblank) lines,
    ; 192 tv picture lines
    ; 30 overscan lines

    ; ref: nick bensema's wonderful "how to draw a playfield"

    ; require 3 VSYNC lines at start of frame
    ; ref: spg pp.1

        lda #D1
        sta VSYNC               ; start vertical synch

    ; there's 76 cycles available on each of these lines
    ; three required for the VSYNC - don't cut it too close
    ; but do interleave other code here

        sta WSYNC               ; 1st
        sta WSYNC               ; 2nd

    ; we have just 37 lines (@2812 cycles) of vblank, so we have to make
    ; sure the screen draw doesn't get stomped on.  the timer will tell
    ; us if we've screwed up.  timer done just before the last WSYNC to
    ; give maximum possible cycle count availability/accuracy

        lda #42                 ; welll... 43.89..
        sta TIM64T              ; we have 37 lines (= 2812 cycles)
                                ; uh... less the 3 for the line below
                            
        sta WSYNC               ; 3rd
        
        lda #0
        sta VSYNC               ; stop vertical synch

    ; screen is now scanning, but tia suppressed
    ; begins again when 0 -> vblank

;--------
; gamecalc

    ; this is running after the TV VSYNC has been sent, but before
    ; we need to start drawing lines.  there are 37 lines, just around
    ; 2800 cycles.  gotta be sure sure that we don't overflow that time
    ; otherwise we'll get screen jitter

gamecalc

    ; game logic goes in here


    ; calculate animation, masking for target display

        inc targettoggle            ; animation for targets
        lda targettoggle
        lsr
        lsr
        and #3*2
        tax

        lda mask,x
        sta targetmask
        lda mask+1,x
        sta targetmask+1


    ; 4 colour playfield support
    ; every alternate frame, the pf colour is changed

    ; colour    what we actually see onscreen
    ;    0        bk1 + bk2
    ;    1        bk1 + pf1
    ;    2        bk2 + pf2
    ;    3        pf1 + pf2     << brightest

    ; there are two phases of display, corresponding to bits in a 2bpp
    ; pixel.  to achieve a colour, make sure it is shown only as in
    ; the list above.

        jsr drawtarget

        lda toggle
        eor #3
        sta toggle

        tax                     ; frame

    ; grab the colours for the coming frame
    ; note, frame = 1 or 2, so we adjust for that

        lda timer
        bpl noflash
        and #$7F
        sta timer               ; D7 of timer causes screen flash

        lda #%11111100
        bne flash

noflash lda bkcol-1,x
flash   sta COLUBK
        lda pfcol1-1,x
        sta COLUP0
        lda pfcol2-1,x
        sta COLUP1

    ; be sure you didn't take too long!

;--------
; drawscreen

    ; the kernel
    ; line by line we have to hold hands with the tia, passing it data
    ; so that it draws the screen for us. good for nostalgia!
    
drawscreen

    ; wait for vertical blank period to end
    ; timer counts down 1 every 64 cycles, so just wait for it
    ; it may already have expired

endvb
        lda INTIM
        bmi endvb

        sta WSYNC               ; not 100% required

        lda #0
        sta VBLANK              ; end vblank period


        ldy #0
predraw sta WSYNC
        iny
        cpy #31
        bne predraw


    ;----
    ; cycle thru drawing line by line...
    ; time is precious here... there are just 228/3 cycles per line
    ; (giving us just 76 cycles) and that's not counting the WSYNC,
    ; should you be writing that. There are 68 pixels (/3 = cycles)
    ; in the hblank period, which starts at the conceptual beginning
    ; of a line on this machine


BLOCKLINE   = 16

        ldx toggle
        lda startx,x
nexlin0 tax

nxtline
    REPEAT 8
        sta WSYNC               ; wait for start of hblank

    ; there are 68 pixels in the hblank period.
    ; we need to have each pf byte written before the scan requires it

    ; NOTE: timing values are possibly out, as its been a while
    ; since I went through and checked them all :)

    ; write left PF as early as we can

        lda #0                  ; 2
        sta PF0                 ; 3/5

        lda playfield,x         ; 4/9
        sta PF1                 ; 3/12          < 28
        lda playfield+1,x       ; 4/16
        sta PF2                 ; 3/19          < 39

        nop                     ; 2/21
        nop                     ; 2/23
        nop                     ; 2/25
        nop                     ; 2/27
        nop                     ; 2/29
        nop                     ; 2/31
        nop                     ; 2/33
        nop                     ; 2/35
        nop                     ; 2/37

        ; right pf

        lda playfield+2,x       ; 4/41
        sta PF0                 ; 3/44          < 49
        lda playfield+3,x       ; 4/47
        sta PF1                 ; 3/50          < 55
        lda playfield+4,x       ; 4/54
        sta PF2                 ; 3/57          < 66
 
        nop                     ; 2/59
        nop                     ; 2/61
        nop                     ; 2/63
 
    ; LINE 2
    ; yeah, ok timing comments are screwed.
    ; YOU fix them :)

        nop
        lda #0
        sta PF0

        lda playfield,x         ; 4/14
        sta PF1                 ; 3/17          < 28
        lda playfield+1,x       ; 4/21
        sta PF2                 ; 3/24          < 39

        nop                     ; 2/26
        nop                     ; 2/28
        nop                     ; 2/30
        nop                     ; 2/32
        nop                     ; 2/34
        nop                     ; 2/36
        nop                     ; 2/38
        nop                     ; 2/40
        nop                     ; 2/42

        ; right pf

        lda playfield+2,x       ; 4/46
        sta PF0                 ; 3/49          < 49
        lda playfield+3,x       ; 4/53
        sta PF1                 ; 3/56          < 55
        lda playfield+4,x       ; 4/60
        sta PF2                 ; 3/63          < 66


    REPEND

        lda next+5,x
        ldx toggle
        cmp endx,x
        bcs nn
        jmp nexlin0
nn

        tya
        clc
        adc #128
        tay

        lda #0
        sta PF0
        sta PF1
        sta PF2



    ; make sure we have a 192 line display
    ; pal must have an even # or the colours screw up?!

        lda #0
fin     sta WSYNC
        iny
        cpy #TOTLINES
        bcc fin

    
        lda #D1
        sta VBLANK              ; end of screen - enter blanking
    
    ; disable all things that could appear onscreen

        lda #0
        sta PF0
        sta PF1
        sta PF2
        sta GRP0
        sta GRP1
        sta ENAM0
        sta ENAM1
        sta ENABL
    
        rts


; tables give indexing based on colour, allowing the
; dual playfields to be displayed alternately

startx  .byte 0,0,PFSIZE,0
endx    .byte 0,PFSIZE,PFSIZE*2,0

;--------
overscan


    ; timer used for delays, whatever
    ; counts 1 each TV frame

        lda timer
        beq notimer
        dec timer
notimer

        jsr moveobjects         ; move before anything is redrawn


    ; after the screen is drawn, we have 30 lines of overscan
    ; this time should be used for game logic

        ldx #30
overlin sta WSYNC
        dex
        bne overlin

        rts


;---------
; drawobject


drawtarget

    ; only draws a single block at a time.
    ; targets can animate, so they're redrawn constantly

        ; draw the targets, using the table

        ldy #LEVEL_BLOCKCOUNT
        lda (level),y

        inc currenttarget
        cmp currenttarget
        bcs drawtargetb
        ldx #0
        stx currenttarget
drawtargetb

        tya
        sec                     ; YES!
        adc currenttarget
        tay
        lda (level),y           ; cell

        tay
        
    ; y = cell number
    ; convert to playfield index, and bitfield index

        ldx celltoplayfield,y
        lda celltobitfield,y
        tay

        lda blockbitpos,y
        eor #$FF
        and playfield,x
        sta temp
        lda blockbitpos,y
        and targetmask
        ora temp
        sta playfield,x

        rts


drawobject

        txa
        pha

    ; pass
    ;    toggle = frame number (1 or 2)
    ;   x       index of object to draw/erase

    ; routine either draws or undraws objects, depending on required
    ; presence in frame buffer.  objects usually remain drawn.
    ; object flag can override and force draw

    ; now we need to add those "just-this-frame" colours
    ; frame        action
    ;    0        only show colour 1 and colour 3 objects
    ;                ie: (un)draw colour 1
    ;    1        only show colour 2 and colour 3 objects
    ;                ie: (un)draw colour 2

        lda objflags,x
        and #3                      ; colour
        sta temp
        tay
        lda startx,y

        ldy objpos,x
                
    ; have determined that draw/erase is required
    ; y = cell number

        clc
        adc celltoplayfield,y
        tax
        lda celltobitfield,y
        tay
        
drawtopf

            ; this will draw in EITHER PF
            ; colour 2 uses PF2

        lda blockbitpos,y
        eor playfield,x
        sta playfield,x

            ; colour 3 requires BOTH playfields
            ; relies on x pointing to PF1

docol3

        lda temp
        cmp #3
        bne done

        lda blockbitpos,y
        eor playfield2,x
        sta playfield2,x
done
        pla
        tax
        rts


;---------
; moveobjects

moveobjects

    ; player movement

        lda #0
        sta SWACNT              ; input

        lda SWCHA               ; joystick
        eor #$FF
        lsr
        lsr
        lsr
        lsr
        tax

        bne nocleartim
        sta timer

nocleartim
        lda timer
        bne nomovep

        lda joystickdir,x
        beq nomovep

        clc
        adc objpos
        sta newpos

        clc
        adc joystickdir,x
        sta newpos+1

        ldy newpos
        jsr checkcell
        beq cellok

    ; we've hit a playfield wall

        lda #D7+5
        sta timer           ; flash!
        bne nomovep

cellok

    ; check to see if we're about to hit another block
    ; if we have, we may be able to push that block
    
        ldy newpos
        jsr checkblock
        beq normalmove

    ; so we've hit another block - lets see if it can move

        ldy newpos+1
        jsr checkcell
        bne nomovep         ; its blocked by a wall

        ldy newpos+1
        jsr checkblock
        bne nomovep         ; blocked by other block
        
    ; maybe we've just homed into a target position

        ldy newpos+1
        jsr checktarget
        bne notspecial

        lda #D7
        sta timer           ; flash!

notspecial

    ; shift the block being pushed

        ldy newpos
        jsr checkblock      ; get index

        jsr drawobject      ; erase
        lda newpos+1
        sta objpos,x        ; move the block
        jsr drawobject      ; draw

normalmove

        lda #MOVEDELAY
        ora timer
        sta timer

        ldx #0
        jsr drawobject      ; erase
        lda newpos
        sta objpos          ; move the player
        ldx #0
        jsr drawobject

nomovep
        rts
    

joystickdir
    .byte 0,-16,16,0,-1,0,0,0,1,0,0,0,0,0,0,0


;--------
; checkcell

checkcell

    ; pass
    ;   y   cell to check
    ; returns
    ;   bne     collision
    ;   beq     NO collision


        sty temp

        lda celltobitfield,y
        ldx celltoplayfield,y
        tay
        
        lda blockbitpos,y
        and bitmap,x
        beq succeed                 ; tis OK as far as playfield

fail    lda #1
        rts

succeed lda #0                      ; no clashes detected
        rts



checktarget

        sty temp

        ldy #LEVEL_BLOCKCOUNT
        lda (level),y
        sta count

scantargets

        iny
        lda (level),y
        cmp temp                ; cell
        beq succeed
        dec count
        bpl scantargets
        bmi fail



;--------
; checkblock

checkblock

    ; pass
    ;   y       cell to check for presence of block
    ; returns
    ;   beq     no hit detected
    ;   +x       index of block hit

        tya

        ldx #OBJSIZE-1
scanblocks
        cmp objpos,x
        beq fail                ; a collision
        dex
        bpl scanblocks
        bmi succeed             ; no collision


; --------
; init1ce

    ; one-time initialization
    ; happens at system reset, no other time

init1ce

        lda #D1
        sta VBLANK              ; start with tia disabled

        lda #D1                 ; reflect / score
        sta CTRLPF

        ldx #0*2                ; level
        lda levelvector,x
        sta level
        lda levelvector+1,x
        sta level+1             ; (level) now vectors to appropriate

    ; copy the playfield from the table
    ; this will be vectored through level, eventually

        ldy #LEVEL_PFVECTOR
        lda (level),y
        sta temp
        iny
        lda (level),y
        sta temp+1              ; PF vector

        ldy #PFSIZE-1
playfilled
        lda (temp),y
        sta playfield,y
        sta playfield2,y        ; maze is C3
        dey
        bpl playfilled


    ; init objects to none

        ldx #OBJSIZE-1
        lda #$80                ; d7 = inactive
initobj sta objpos,x
        dex
        bpl initobj


    ; add a couple of test objects

        ldx #OBJSIZE-1
seti    lda tempinit,x
        sta objpos,x
        lda #2
        sta objflags,x
        jsr drawobject
        dex
        bne seti


        lda #27+64
        sta objpos
        lda #1
        sta objflags
        ldx #0
        jsr drawobject

        lda #1
        sta toggle              ; cycles 1/2/1/2...

        lda #0
        sta timer


    ; at this stage we haven't drawn colour 3 objects
    ; that comes later - give it time :)

        rts



tempinit

    .byte 34,42+16,33+16,37+48,38+64,39+48
    
bitmap

; in pf0 and pf2, the most significant bit (d7) is on the right
; side.  in pf1, the most significant bit is on the left side.  this
; means that relative to pf0 and pf2, pf1 has a reversed bit order.
;
;    pf0  |     pf1       |      pf2
;  4 5 6 7|7 6 5 4 3 2 1 0|0 1 2 3 4 5 6 7

    ; just treat this as a 40x24 bitmap
    ; note: left P0 is hardwired at draw-time

    ;     pf1 -->   pf2 <--   pf0 <--   pf1 -->   pf2 <--
    ;     ok        flip      flip xxxx ok        flip
    .byte %11111111,%11111111,%11110000,%11111111,%00001111
    .byte %11001100,%00001100,%00110000,%00000000,%00001100
    .byte %11000000,%00000000,%00000000,%11001100,%00001111
    .byte %11001111,%11110011,%00000000,%00000000,%00001100
    .byte %11000000,%00110011,%11000000,%00111100,%00001111
    .byte %11001100,%00110011,%00000000,%00000000,%00001100
    .byte %11110000,%00000000,%11000000,%00111100,%00001100
    .byte %11000000,%00000000,%00110000,%00000000,%00001100
    .byte %11111111,%11111111,%11110000,%11111111,%00001111
    ;                              ^^^^
    ;                             unused


    ; colours for 4-colour screen
    ; these colours correspond to each of 2 frames of the multiplexed
    ; display.  there are two bkcols available, and two sides of the screen
    ; the 4 colours are from combination of the two frame multiplex, and
    ; the combination display on these frames.  colours are combiniations
    ; hopefully this won't flicker to buggery on the real thing, like it
    ; does on my emulator!

    ;               fr.1        fr.2
bkcol     .byte    %10000100,  %10000100
pfcol1    .byte    %00010100,  %00111000
pfcol2    .byte    %00010100,  %00111000


    ; draw masks for block display
    ; index by x coordinate (0-38)
    ; caters for each position possible in the 6 byte wide display
    ; these values can be eor-drawn direct into the playfield array

    ; x     = crosses bytes (2 bytes needed to draw)
    ; [n]   = x pos

blockbitpos

    ;     /  left screen   \  /    right side of screen   \
    ;     PF1       PF2       PF0       PF1       PF2
    ;     -->       <--       <--  xxxx -->       <--
    .byte %11000000,%00000000,%00000000,%00000000,%00000000
    .byte %01100000,%00000000,%00000000,%00000000,%00000000
    .byte %00110000,%00000000,%00000000,%00000000,%00000000 ;10
    .byte %00011000,%00000000,%00000000,%00000000,%00000000
    .byte %00001100,%00000000,%00000000,%00000000,%00000000 ;20
    .byte %00000110,%00000000,%00000000,%00000000,%00000000
    .byte %00000011,%00000000,%00000000,%00000000,%00000000 ;30
    .byte %00000001,%00000001,%00000000,%00000000,%00000000
    .byte %00000000,%00000011,%00000000,%00000000,%00000000 ;40
    .byte %00000000,%00000110,%00000000,%00000000,%00000000
    .byte %00000000,%00001100,%00000000,%00000000,%00000000 ;50
    .byte %00000000,%00011000,%00000000,%00000000,%00000000
    .byte %00000000,%00110000,%00000000,%00000000,%00000000 ;60
    .byte %00000000,%01100000,%00000000,%00000000,%00000000
    .byte %00000000,%11000000,%00000000,%00000000,%00000000 ;70
    .byte %00000000,%10000000,%00010000,%00000000,%00000000
    .byte %00000000,%00000000,%00110000,%00000000,%00000000 ;80
    .byte %00000000,%00000000,%01100000,%00000000,%00000000
    .byte %00000000,%00000000,%11000000,%00000000,%00000000 ;90
    .byte %00000000,%00000000,%10000000,%10000000,%00000000
    .byte %00000000,%00000000,%00000000,%11000000,%00000000 ;100
    .byte %00000000,%00000000,%00000000,%01100000,%00000000
    .byte %00000000,%00000000,%00000000,%00110000,%00000000 ;110
    .byte %00000000,%00000000,%00000000,%00011000,%00000000
    .byte %00000000,%00000000,%00000000,%00001100,%00000000 ;120
    .byte %00000000,%00000000,%00000000,%00000110,%00000000
    .byte %00000000,%00000000,%00000000,%00000011,%00000000 ;130
    .byte %00000000,%00000000,%00000000,%00000001,%00000001
    .byte %00000000,%00000000,%00000000,%00000000,%00000011 ;140
    .byte %00000000,%00000000,%00000000,%00000000,%00000110
    .byte %00000000,%00000000,%00000000,%00000000,%00001100 ;150
    .byte %00000000,%00000000,%00000000,%00000000,%00011000

mask
    .byte %10101010,%01010101
    .byte %01010101,%10101010
    .byte %10101010,%01010101
    .byte %01010101,%10101010


next
x   set 0
    repeat 256
    .byte x
x   set x+1
    repend
nextend


; levels

levelvector
    .word level0
    ;...etc

    ; format of levels
    ; byte          number of targets - 1
    ; byte [n]      position of each target
    ; byte [n]      init position for each block


LEVEL_PFVECTOR      = 0
LEVEL_BLOCKCOUNT    = 2


level0
    .word bitmap       ; background pointer
    .byte 4            ; # targets -1
    .byte 26+16*6,30,17,38,30+16*6
                ; target index
    .byte 1            ; unpositioned block


celltoplayfield

    .byte 0,0,0,0,1,1,1,1,2,2,3,3,3,3,4,4
    .byte 5,5,5,5,6,6,6,6,7,7,8,8,8,8,9,9
    .byte 10,10,10,10,11,11,11,11,12,12,13,13,13,13,14,14
    .byte 15,15,15,15,16,16,16,16,17,17,18,18,18,18,19,19
    .byte 20,20,20,20,21,21,21,21,22,22,23,23,23,23,24,24
    .byte 25,25,25,25,26,26,26,26,27,27,28,28,28,28,29,29
    .byte 30,30,30,30,31,31,31,31,32,32,33,33,33,33,34,34
    .byte 35,35,35,35,36,36,36,36,37,37,38,38,38,38,39,39
    .byte 40,40,40,40,41,41,41,41,42,42,43,43,43,43,44,44
    .byte 45,45,45,45,46,46,46,46,47,47,48,48,48,48,49,49

celltobitfield

    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154
    .byte   0, 10, 20, 30, 41, 51, 61, 71, 82, 92,103,113,123,133,144,154

;--------
; system vectors

    org $FFFC
    .word start                 ; irq
    .word start                 ; reset vector

;EOF